/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 19 janvier 2007
   derniere modification le 20/03/2008 */

#ifndef SCESCENE_H
#define SCESCENE_H

#include <SCE/interface/SCECamera.h>
#include <SCE/interface/SCENode.h>
#include <SCE/interface/SCEModel.h>

/*
  Gestionnaire de scenes.
  
  Morceau principal du moteur, gestion des objets dans la scene:
  - octree/quadtree
  - BSP
  - frustum culling
  - contribution culling
  - modeles 3D --> SCEModel
  - cameras
    - mouvements avances
    - multiples
    - matrice de projection
  - lumieres
  - interactions:
    - objets/lumieres
    - camera/lumieres (occlusion lighting, cf occlusion queries)
  - ajouts d'objets
  - ajouts de lumieres
  - changement d'etats:
    - changement des arbres de partitionnement a utiliser --> BSP, quadtree, ...
    - effet global a la scene --> HDR, bloom, glow, ...
    - fonctionnalites divers...
  - LOD --> textures, modeles, terrain, ...
  - tri/ordre d'affichage:
    - textures
    - shaders
    - modeles
    - materiaux
  - sauvegarde d'un effet special definit par
    l'utilisateur et utilisation a la demande
  - screenshot --> DevIL pour l'enregistrement
  - video --> Open*L... a voir sur dvp.net (OpenCV .. ?)
  - rendu multi-passes --> HDR, bloom, post-effect, ...
  - IA de traitement des donnees (cf ordre d'affichage)
  - {occlusion culling
  - {occlusion queries
  - scene hierarchique ...
  - ...
*/

typedef struct sce_sscene SCE_SScene;
struct sce_sscene
{
    SCE_SNode *rootnode; /* noeud racine */

    SCE_SList *models;   /* liste des modeles de la scene */
    SCE_SList *lights;   /* liste des lumieres de la scene */
    SCE_SList *cameras;  /* liste des cameras actives de la scene */

    int frustum_culling; /* booleen definissant l'usage du frustum culling */
};

/* initialise le gestionnaire de scenes */
int SCE_Init_Scene (void);
/* quitte le gestionnaire de scenes */
void SCE_Quit_Scene (void);

/* cree une nouvelle scene */
SCE_SScene* SCE_Scene_Create (void);
/* supprime une scene */
void SCE_Scene_Delete (SCE_SScene*);

/* determine l'etat du frustum culling */
void SCE_Scene_EnableFrustumCulling (SCE_SScene*);
void SCE_Scene_DisableFrustumCulling (SCE_SScene*);

/* genere les boundings boxes de tous les modeles de la scene */
void SCE_Scene_GenerateBoundingsBoxes (SCE_SScene*);

/* retourne le noeud racine */
SCE_SNode* SCE_Scene_GetRootNode (SCE_SScene*);
/* retourne la camera specifiee */
SCE_SCamera* SCE_Scene_GetCamera (SCE_SScene*, unsigned int);

/* ajoute un noeud au noeud racine */
int SCE_Scene_AddNode (SCE_SScene*, SCE_SNode*);
/* ajoute un modele a une scene */
int SCE_Scene_AddModel (SCE_SScene*, SCE_SModel*);
/* ajoute une lumiere a une scene */
int SCE_Scene_AddLight (SCE_SScene*, SCE_SLight*);
/* ajoute une camera a une scene */
int SCE_Scene_AddCamera (SCE_SScene*, SCE_SCamera*);

/* rend une scene */
void SCE_Scene_Render (SCE_SScene*);

#endif /* guard */
